@@ -6,80 +6,63 @@ Contains the following classes:
     RobotLogCallback Class: Collects robot training statistics
     RobotEvalCallback Class: Evaluates robot model every certain number of steps
 """
-import numpy as np
+from typing import Optional
 import time
+import numpy as np
+from torch import nn
+import gym
 from pandas import DataFrame, concat
 from stable_baselines3.common.callbacks import BaseCallback
 from stable_baselines3.common.vec_env.base_vec_env import VecEnv
 
 
+THINK_EMOJI = "\U0001F914"
+
+
 class RobotMaxStepsCallback(BaseCallback):
-    """
-    A custom callback that derives from ``BaseCallback``.
+    """Custom callback that derives from ``BaseCallback``"""
 
-    :param verbose: Verbosity level: 0 for no output, 1 for info messages, 2 for debug messages
-    """
-
-    def __init__(self, max_steps=1e6, verbose=0):
-        super(RobotMaxStepsCallback, self).__init__(verbose)
+    def __init__(self, max_steps: int = int(1e6), verbose: int = 0):
+        super().__init__(verbose)
         self.max_steps = max_steps
 
-    def _on_training_start(self) -> None:
-        """
-        This method is called before the first rollout starts.
-        """
-        pass
-
-    def _on_rollout_start(self) -> None:
-        """
-        A rollout is the collection of environment interaction
-        using the current policy.
-        This event is triggered before collecting new samples.
-        """
-        pass
-
     def _on_step(self) -> bool:
-        """
-        This method will be called by the model after each call to `env.step()`.
+        """This method will be called by the model after each call to `env.step()`.
 
         For child callback (of an `EventCallback`), this will be called
         when the event is triggered.
 
-        :return: (bool) If the callback returns False, training is aborted early.
+        Returns:
+            bool: If the callback returns False, training is aborted early.
         """
-        thinking_emoji = "\U0001F914"
         total_steps = self.training_env.get_attr(attr_name="total_steps")[0]  # type: ignore
+
         if total_steps >= self.max_steps:
             print(
-                f"{thinking_emoji} {thinking_emoji} Abort_training {thinking_emoji} {thinking_emoji}"
+                f"{THINK_EMOJI} {THINK_EMOJI} Abort_training {THINK_EMOJI} {THINK_EMOJI}"
             )
             return False
         return True
 
-    def _on_rollout_end(self) -> None:
-        """
-        This event is triggered before updating the policy.
-        """
-        pass
-
-    def _on_training_end(self) -> None:
-        """
-        This event is triggered before exiting the `learn()` method.
-        """
-        pass
-
 
 class RobotSuccessesCallback(BaseCallback):
-    def __init__(self, num_successes=5, verbose=0):
-        super(RobotSuccessesCallback, self).__init__(verbose)
+    """Callback for checking for the number of successes"""
+
+    def __init__(self, num_successes: int = 5, verbose: int = 0):
+        super().__init__(verbose)
         self.num_successes = num_successes
 
     def _on_step(self) -> bool:
-        thinking_emoji = "\U0001F914"
-        total_num_successes = self.training_env.get_attr(attr_name="num_successes")[0]  # type: ignore
+        """Check if robot reached desired number of successes
+
+        Returns:
+            bool: Flag for aborting training early
+        """
+        attribute_name = "num_successes"
+        total_num_successes = self.training_env.get_attr(attribute_name)[0]  # type: ignore
         if total_num_successes >= self.num_successes:
             print(
-                f"{thinking_emoji} {thinking_emoji} Abort_training {thinking_emoji} {thinking_emoji}"
+                f"{THINK_EMOJI} {THINK_EMOJI} Abort_training {THINK_EMOJI} {THINK_EMOJI}"
             )
             return False
         return True
@@ -95,18 +78,18 @@ class RobotLogCallback(BaseCallback):
     def __init__(
         self,
         train_env,
-        logpath: str = None,
+        logpath: Optional[str] = None,
         eval_frequency: int = 10000,
         verbose: int = 0,
     ) -> None:
-        super(RobotLogCallback, self).__init__(verbose)
+        super().__init__(verbose)
         self.training_env = train_env
         self.logpath = logpath
         self.eval_frequency = eval_frequency
         self.last_len_statistics = 0
         self.best_avg_reward = [-np.inf]
         self.last_eval_time = time.time()
-        self.total_time = 0
+        self.total_time:float = 0
 
     def _on_step(self) -> bool:
         """print statistics and save training logs every eval_frequency timesteps.
@@ -132,7 +115,9 @@ class RobotLogCallback(BaseCallback):
             save_logs(train_logs, self.logpath, self.verbose)
             if self.verbose:
                 print_statistics_for_n_steps(
-                    last_added_train_logs, elapsed=elapsed, n=10000
+                    last_added_train_logs,
+                    elapsed_time=elapsed,
+                    num_steps=10000,
                 )
             if self.n_calls % (self.eval_frequency * 10) == 0 and self.verbose:
                 print_statistics_train(
@@ -148,7 +133,8 @@ class RobotLogCallback(BaseCallback):
 
 
 class RobotEvalCallback(BaseCallback):
-    """Runs evaluation episodes on the trained model,save the evaluation logs and saves the model if improved in evaluation.
+    """Runs evaluation episodes on the trained model,save the evaluation logs
+    and saves the model if improved in evaluation.
 
     Args:
         BaseCallback (Class): Base class for callback in stable_baselines3.
@@ -156,15 +142,15 @@ class RobotEvalCallback(BaseCallback):
 
     def __init__(
         self,
-        eval_env,
+        eval_env: gym.Env,
         n_eval_episodes=100,
-        logpath: str = None,
-        savepath: str = None,
+        logpath: Optional[str] = None,
+        savepath: Optional[str] = None,
         eval_frequency: int = 50000,
         verbose: int = 1,
         render=False,
     ) -> None:
-        super(RobotEvalCallback, self).__init__(verbose)
+        super().__init__(verbose)
 
         self.logpath = logpath
         self.n_eval_episodes = n_eval_episodes
@@ -177,7 +163,8 @@ class RobotEvalCallback(BaseCallback):
         self.render = render
 
     def _on_step(self) -> bool:
-        """Runs evaluation episodes on the trained model every eval_frequency timesteps, saves the evaluation logs and saves the model if improved in evaluation.
+        """Runs evaluation episodes on the trained model every eval_frequency timesteps,
+        saves the evaluation logs and saves the model if improved in evaluation.
 
         Returns:
             bool: If the callback returns False, training is aborted early.
@@ -200,113 +187,121 @@ class RobotEvalCallback(BaseCallback):
         return True
 
     def save_model_if_improved(
-        self, new_avg_reward: float, model, savepath: str
+        self,
+        new_avg_reward: float,
+        model: nn.Module,
+        savepath: Optional[str],
     ) -> None:
-        """saves the model if the average reward improved in evaluation.
+        """Save the model if the average reward improved in evaluation.
 
         Args:
-            new_avg_reward (float): the new average reward in evaluation.
-            model (_type_): trained model.
-            savepath (str): path to save the model to it.
+            new_avg_reward (float): New average reward in evaluation
+            model (_type_): Trained model instance
+            savepath (str): Path to save the model
         """
+        if new_avg_reward <= self.best_avg_reward[0]:
+            return
 
-        if new_avg_reward > self.best_avg_reward[0]:
-            self.best_avg_reward[0] = new_avg_reward
-            if savepath is not None:
-                try:
-                    model.save(savepath)
-                    print(
-                        "model saved to {} (avg reward: {}).".format(
-                            savepath, new_avg_reward
-                        )
-                    )
-                except:
-                    print("Could not save")
+        self.best_avg_reward[0] = new_avg_reward
+        if savepath is not None:
+            try:
+                model.save(savepath)
+                print(f"Model saved to {savepath} (avg reward: {new_avg_reward}).")
+            except AttributeError:
+                print("Could not save")
+            else:
+                print("An error occured while saving the model")
 
 
-def save_logs(training_logs: DataFrame, logpath: str, verbose: int) -> None:
+def save_logs(training_logs: DataFrame, logpath: Optional[str], verbose: int) -> None:
     """Saves the training logs of the robot.
 
     Args:
-        training_logs (DataFrame): contains robot training logs.
-        logpath (str): path to save the logs to it.
-        verbose (int): flag to print the saving path.
+        training_logs (DataFrame): Contains robot training logs.
+        logpath (str): Path to save the logs to it.
+        verbose (int): Flag to print the saving path.
     """
-    if logpath is not None:
-        training_logs.to_csv(logpath)
-        if verbose > 1:
-            print("log saved to {}.".format(logpath))
+    training_logs.to_csv(logpath)
+    if verbose > 1:
+        print(f"Logs saved to {logpath}")
 
 
 def print_statistics_train(
-    train_logs: DataFrame, elapsed: float, n_calls: int, n_train_steps: int
+    train_logs: DataFrame,
+    elapsed_time: float,
+    n_calls: int,
+    n_train_steps: int,
 ) -> None:
-    """print statistics of training.
+    """Print statistics of training.
 
 
     Args:
-        train_logs (DataFrame): contains robot training logs.
-        elapsed (float): time elapsed in training.
-        n_calls (int): number of training steps per env.
-        n_train_steps (int): total number of training steps.
+        train_logs (DataFrame): Contains robot training logs.
+        elapsed (float): Time elapsed in training.
+        n_calls (int): Number of training steps per env.
+        n_train_steps (int): Total number of training steps.
     """
     scenarios = sorted(list(set(train_logs["scenario"].values)))
     rewards = train_logs["reward"].values
-    print("Statistics for {:10d} steps".format(n_train_steps))
-    print(
-        "Steps Per Env = {:5d} ,Total Steps = {:5d} ,Total episodes = {:5d} done in {} sec  ".format(
-            n_calls, n_train_steps, len(train_logs), elapsed
-        )
-    )
+    print(f"Statistics for {n_train_steps:10d} steps")
+    num_episodes = len(train_logs)
+    stats = f"Steps Per Env = {n_calls:5d} ,"
+    stats += f"Total Steps = {n_train_steps:5d} ,"
+    stats += f"Total episodes = {num_episodes:5d}"
+    stats += f"done in {elapsed_time} sec"
+    print(stats)
     for scenario in scenarios:
         is_scenario = train_logs["scenario"].values == scenario
         scenario_rewards = rewards[is_scenario]
-        print(
-            "Reward in training : {:.4f} ({})".format(
-                np.mean(scenario_rewards), len(scenario_rewards)
-            )
-        )
+        avg_scenario_rewards = np.mean(scenario_rewards).item()
+        num_scenarios = len(scenario_rewards)
+        print(f"Avg rewards (train): {avg_scenario_rewards:.4f} ({num_scenarios}")
 
 
 def print_statistics_for_n_steps(
-    logs: DataFrame, elapsed: float, n: int = 10000
+    logs: DataFrame,
+    elapsed_time: float,
+    num_steps: int = 10000,
 ) -> None:
-    """_summary_
+    """Print training stats(`num_eposides`, `avg_rewards`)
+    for n steps.
 
     Args:
-        logs (DataFrame): contains robot training logs for the last n timesteps.
-        elapsed (float): time elapsed in last n training timesteps.
-        n (int, optional): number of training steps to print. Defaults to 10000.
+        logs (DataFrame): Contains training logs for the last n timesteps.
+        elapsed_time (float): Time elapsed in last n training timesteps.
+        n (int, optional): Number of training steps to print. Defaults to 10000.
     """
     scenarios = sorted(list(set(logs["scenario"].values)))
     rewards = logs["reward"].values
     # print statistics
-
-    print("Statistics for last {} steps".format(n))
-    print("Total episodes = {:5d} done in {} sec ".format(len(logs), elapsed))
+    print(f"Statistics for last {num_steps} steps")
+    num_eposides = len(logs)
+    print(f"Total episodes = {num_eposides:5d} done in {elapsed_time} secs")
     for scenario in scenarios:
         is_scenario = logs["scenario"].values == scenario
         scenario_rewards = rewards[is_scenario]
-        print(
-            "Reward in training : {:.4f} ({})".format(
-                np.mean(scenario_rewards), len(scenario_rewards)
-            )
-        )
+        avg_scenario_rewards = np.mean(scenario_rewards).item()
+        num_scenarios = len(scenario_rewards)
+        print(f"Avg reward (train): {avg_scenario_rewards:.4f} ({num_scenarios})")
 
 
-def run_n_episodes(model, env, n: int) -> DataFrame:
-    """runs n evaluation episodes on the trained model.
+def run_n_episodes(
+    model: nn.Module,
+    env: gym.Env,
+    num_eposides: int,
+) -> DataFrame:
+    """Run n evaluation-episodes on the trained model.
 
     Args:
-        model (_type_): trained model.
-        env (_type_): evaluation env.
-        n (int): number of episodes.
+        model (torch.nn.Module): Trained model.
+        env (gym.Env): Evaluation env.
+        num_eposides (int): Number of episodes to evaluate on.
 
     Returns:
-        DataFrame: logs for evaluation episodes.
+        DataFrame: Logs for evaluation episodes.
     """
     env.episode_statistics["scenario"] = "robot_env_test"
-    for i in range(n):
+    for _ in range(num_eposides):
         obs = env.reset()
         done = False
         while not done:
@@ -315,8 +310,11 @@ def run_n_episodes(model, env, n: int) -> DataFrame:
     return env.episode_statistics
 
 
-def print_statistics_eval(eval_logs: DataFrame, eval_elapsed: float) -> None:
-    """print statistics of evaluation.
+def print_statistics_eval(
+    eval_logs: DataFrame,
+    eval_elapsed: float,
+) -> None:
+    """Print statistics of evaluation
 
     Args:
         eval_logs (DataFrame): ontains robot evaluation logs.
@@ -324,12 +322,10 @@ def print_statistics_eval(eval_logs: DataFrame, eval_elapsed: float) -> None:
     """
     scenarios = sorted(list(set(eval_logs["scenario"].values)))
     rewards = eval_logs["reward"].values
-    print("Evaluation time : {} ".format(eval_elapsed))
+    print(f"Evaluation time : {eval_elapsed} ")
     for scenario in scenarios:
         is_scenario = eval_logs["scenario"].values == scenario
         scenario_rewards = rewards[is_scenario]
-        print(
-            "{}: {:.4f} ({})".format(
-                scenario, np.mean(scenario_rewards), len(scenario_rewards)
-            )
-        )
+        avg_scenario_rewards = np.mean(scenario_rewards).item()
+        num_scenarios = len(scenario_rewards)
+        print(f"{scenario}: {avg_scenario_rewards:.4f} ({num_scenarios})")
