@@ -23,6 +23,8 @@ from highrl.configs.colors import (
     nosecolor,
     agentcolor,
 )
+from torch.utils.tensorboard import SummaryWriter
+
 from highrl.obstacle.single_obstacle import SingleObstacle
 from highrl.utils.action import ActionXY
 from highrl.utils.calculations import point_to_point_distance
@@ -73,7 +75,9 @@ class RobotEnv(Env):
         self.robot_goal_py: int = 0
 
         self.is_initial_state = True
-
+        self.tensorboard_dir = "runs/robot"
+        self.rwrd_grph_name = "reward"
+        self.eps_rwrd_grph_name = "episode_reward"
         # Results of each episode
         # Contains [episode_reward, episode_steps, success_flag]
         self.results: List[Tuple[float, int, bool]] = []
@@ -101,6 +105,7 @@ class RobotEnv(Env):
                     "wall_time",
                 ]
             )
+        self.robot_writer = SummaryWriter(log_dir=self.tensorboard_dir)
 
     def _configure(self, config: configparser.RawConfigParser) -> None:
         """Configure environment variables using input config object
@@ -168,6 +173,16 @@ class RobotEnv(Env):
         )
 
         self.episode_reward += self.reward
+        self.robot_writer.add_scalar(
+            self.rwrd_grph_name,
+            self.reward,
+            self.total_steps,
+        )
+        self.robot_writer.add_scalar(
+            self.eps_rwrd_grph_name,
+            self.episode_reward,
+            self.total_steps,
+        )
 
         if self.episode_steps % self.render_each == 0:
             self.render(save_to_file=self.save_to_file)
